package xyz.lychee.lagfixer.libs.adventure.identity;

import java.util.Locale;
import java.util.UUID;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.pointer.Pointer;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

public interface Identity extends Examinable, Identified {
   Pointer<String> NAME = Pointer.pointer(String.class, Key.key("adventure", "name"));
   Pointer<UUID> UUID = Pointer.pointer(UUID.class, Key.key("adventure", "uuid"));
   Pointer<Component> DISPLAY_NAME = Pointer.pointer(Component.class, Key.key("adventure", "display_name"));
   Pointer<Locale> LOCALE = Pointer.pointer(Locale.class, Key.key("adventure", "locale"));

   static @NotNull Identity nil() {
      return NilIdentity.INSTANCE;
   }

   static @NotNull Identity identity(final @NotNull UUID uuid) {
      return (Identity)(uuid.equals(NilIdentity.NIL_UUID) ? NilIdentity.INSTANCE : new IdentityImpl(uuid));
   }

   @NotNull UUID uuid();

   default @NotNull Identity identity() {
      return this;
   }

   default @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("uuid", this.uuid()));
   }
}
